/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package test;

import java.util.logging.Level;
import java.util.logging.Logger;
import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import javax.ejb.EJBException;
import javax.ejb.LocalBean;
import javax.ejb.Stateless;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.TextMessage;

/**
 *
 * @author ndeakin
 */
@Stateless
@LocalBean
public class QueueSenderBean {
    @Resource(mappedName = "jms/txTestQueue")
    private Queue txTestQueue;
    @Resource(mappedName = "jms/txTestConnectionFactory")
    private ConnectionFactory connectionFactory;
    
    private Connection connection;
    private Session session;
    private MessageProducer messageProducer;
    
    private static final Logger logger = Logger.getLogger(QueueSenderBean.class.getName());
    
    @PostConstruct
    private void initJmsResources() {
        try {
            connection = connectionFactory.createConnection();
            session = connection.createSession(true, Session.AUTO_ACKNOWLEDGE);
            messageProducer = session.createProducer(txTestQueue);
        } catch (JMSException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
            throw new EJBException(ex);
        }
    }

    @PreDestroy
    private void closeJmsResources() {
        try {
            if (messageProducer != null)
                messageProducer.close();
            
            if (session != null)
                session.close();
            
            if (connection != null)
                connection.close();
            
        } catch (JMSException e) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "closeJmsResources()", e);
        } finally {
            messageProducer = null;
            session = null;
            connection = null;
        }
    }    

    public void sendMessage(String text) {
        
       logger.log(Level.INFO,"QueueSenderBean.sendMessage() entry: instance is "+this.hashCode());
       //initJmsResources(); 
       try {
            TextMessage jmsMessage = session.createTextMessage();
            jmsMessage.setText(text);
            messageProducer.send(jmsMessage);
        } catch (JMSException ex) {
            Logger.getLogger(getClass().getName()).log(Level.SEVERE, "sendJMSMessageToTxTestQueue()", ex);
            throw new EJBException(ex);
        } finally {
            //closeJmsResources();
            logger.log(Level.INFO,"QueueSenderBean.sendMessage() exit: instance is "+this.hashCode());
        }
    }
    
}
