/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package test;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.QueueBrowser;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.jms.TopicConnectionFactory;
import javax.management.JMException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.transaction.UserTransaction;

public class TransactionTestServlet extends HttpServlet {
    @EJB
    private TransactionTestSB transactionTestSB;
    
    public static int CFTYPE_QUEUE_CONNECTION = 1;
    public static int CFTYPE_TOPIC_CONNECTION = 2;
    public static int CFTYPE_CONNECTION = 3;
    
    @Resource(mappedName = "jms/txTestQueueConnectionFactory")
    private QueueConnectionFactory txTestQueueConnectionFactory;
    @Resource(mappedName = "jms/txTestTopicConnectionFactory")
    private TopicConnectionFactory txTestTopicConnectionFactory;
    @Resource(mappedName = "jms/txTestConnectionFactory")
    private ConnectionFactory txTestConnectionFactory;
    @Resource(mappedName = "jms/txTestQueue")
    private Queue txTestQueue;
    @Resource(mappedName = "jms/txTestTopic")
    private Topic txTestTopic;    
    @Resource(mappedName = "java:comp/UserTransaction")
    private UserTransaction userTransaction;
    
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        response.setContentType("text/html;charset=UTF-8");
        PrintWriter out = response.getWriter();
        try {
            String testStr = (String)request.getParameter("test");
            String cftypeStr = (String)request.getParameter("cftype");
            out.println("<html>");
            out.println("<head>");
            out.println("<title>Servlet TransactionTestServlet</title>");  
            out.println("</head>");
            out.println("<body>");
            out.println("<h1>Servlet TransactionTestServlet at " + request.getContextPath () + "</h1>");
            int testNum=Integer.parseInt(testStr);
            int cfType = validateCFType(cftypeStr);
            
            out.println("Test case = "+testNum+" starting<br>");
            switch (testNum) {
                case 1:  test1(out,cfType); break;
                case 2:  test2(out,cfType); break;
                case 3:  test3(out,cfType); break;     
                case 4:  test4(out,cfType); break;                       
            default: System.out.println("Invalid test case "+testNum);break;
            }
            out.println("Test case = "+testNum+" successfully completed<br>");

        } catch (Exception e){
            out.println("<p>Something bad happened. Here's what I know:</p>");
            out.println("<pre>");
            e.printStackTrace(out);
           out.println("</pre>");
        }
        out.println("</body>");
        out.println("</html>");
        out.close();
    } 
    
    private int validateCFType(String cftype) throws Exception {
            
        if (cftype==null) {
            throw new Exception("Parameter cftype is not set. Must be Connection, QueueConnection or TopicConnection");
        }
        
        if (cftype.equalsIgnoreCase("QueueConnection")) return CFTYPE_QUEUE_CONNECTION;
        if (cftype.equalsIgnoreCase("TopicConnection")) return CFTYPE_TOPIC_CONNECTION;
        if (cftype.equalsIgnoreCase("Connection")) return CFTYPE_CONNECTION;
        
        throw new Exception("Parameter cftype is invalid. Must be Connection, QueueConnection or TopicConnection");
    }

    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /** 
     * Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        processRequest(request, response);
    } 

    /** 
     * Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        processRequest(request, response);
    }

    /** 
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>

    private void test1(PrintWriter out, int cftype) throws Exception {
        
        int depthBefore = getQueueDepth(txTestQueue);

        userTransaction.begin();
        Connection connection = txTestConnectionFactory.createConnection();
        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
        MessageProducer producer = session.createProducer(txTestQueue);
        TextMessage message = session.createTextMessage("Here is the news");
        producer.send(message);
            
        assertEquals("Message sent but not committed", depthBefore, getQueueDepth(txTestQueue));
        userTransaction.commit();
        assertEquals("Message committed", depthBefore+1, getQueueDepth(txTestQueue));
       
        connection.close();

    }
    
        private void test2(PrintWriter out, int cftype) throws Exception {
        
        int depthBefore = getQueueDepth(txTestQueue);
        out.print("Depth at start of test="+getQueueDepth(txTestQueue)+"<br>");
               
        Connection connection = txTestConnectionFactory.createConnection();
        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
        MessageProducer producer = session.createProducer(txTestQueue);
        TextMessage message = session.createTextMessage("Here is the news");
        producer.send(message);
        
        // depth should be incremented as transaction is non-transacted +1
        assertEquals("Message autocommitted", depthBefore+1, getQueueDepth(txTestQueue));
        out.print("Depth 1 ="+getQueueDepth(txTestQueue)+"Should be "+(depthBefore+1)+"<br>");
        userTransaction.begin();
        producer.send(message);
        // depth should be unchanged as transaction is not committed +1
        out.print("Depth 2 ="+getQueueDepth(txTestQueue)+"Should be "+(depthBefore+1)+"<br>");
        assertEquals("Message sent but not committed", depthBefore+1, getQueueDepth(txTestQueue));           
        userTransaction.commit();
        // depth should be incremented as transaction has been committed +2
        out.print("Depth 3 ="+getQueueDepth(txTestQueue)+"Should be "+(depthBefore+2)+"<br>");
        assertEquals("Message committed", depthBefore+2, getQueueDepth(txTestQueue));   
         
//        producer.send(message);
//        // depth should be incremented as transaction is non-transacted +3
//        out.print("Depth 4 ="+getQueueDepth(txTestQueue)+"Should be "+(depthBefore+3)+"<br>");
//        assertEquals("Message  sent and autocommitted", depthBefore+3, getQueueDepth(txTestQueue));
        connection.close();
    }
        
    private void test3(PrintWriter out, int cftype) throws Exception {
        
        int depthBefore = getQueueDepth(txTestQueue);
        out.print("Depth at start of test="+getQueueDepth(txTestQueue)+"<br>");
               
        Connection connection = txTestConnectionFactory.createConnection();
        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
        MessageProducer producer = session.createProducer(txTestQueue);
        TextMessage message = session.createTextMessage("Here is the news");
        producer.send(message);
        
        // depth should be incremented as transaction is non-transacted +1
        assertEquals("Message autocommitted", depthBefore+1, getQueueDepth(txTestQueue));
        out.print("Depth 1 ="+getQueueDepth(txTestQueue)+"Should be "+(depthBefore+1)+"<br>");
        userTransaction.begin();
        producer.send(message);
        // depth should be unchanged as transaction is not committed +1
        out.print("Depth 2 ="+getQueueDepth(txTestQueue)+"Should be "+(depthBefore+1)+"<br>");
        assertEquals("Message sent but not committed", depthBefore+1, getQueueDepth(txTestQueue));           
        userTransaction.rollback();
        // depth should be unchanged as transaction was tolled back +1
        out.print("Depth 3 ="+getQueueDepth(txTestQueue)+"Should be "+(depthBefore+1)+"<br>");
        assertEquals("Message rolled back", depthBefore+1, getQueueDepth(txTestQueue));   
         
//        producer.send(message);
//        // depth should be incremented as transaction is non-transacted +2
//        out.print("Depth 4 ="+getQueueDepth(txTestQueue)+"Should be "+(depthBefore+2)+"<br>");
//        assertEquals("Message  sent and autocommitted", depthBefore+2, getQueueDepth(txTestQueue));
        connection.close();
    }        
    
    private void test4(PrintWriter out, int cftype) throws Exception {
        
        transactionTestSB.sendMessageCMT();
        
    }

//    private int getDestinationDepth(int port, String destName) throws IOException, JMSException, JMException {
//
//		AdminConnectionFactory acf = new AdminConnectionFactory();
//		acf.setProperty(AdminConnectionConfiguration.imqAddress, "localhost:" + port);
//		JMXConnector jmxc1 = acf.createConnection();
//		MBeanServerConnection mbsc = jmxc1.getMBeanServerConnection();
//
//		ObjectName bkrCfg = new ObjectName(MQObjectName.BROKER_CONFIG_MBEAN_NAME);
//
//		// Setup parameters and signature for getProperty operation.
//		Object params[] = { propName };
//		String signature[] = { String.class.getName() };
//
//		String actualValue = (String) mbsc.invoke(bkrCfg, BrokerOperations.GET_PROPERTY, params, signature);
//		assertNotNull(message + " Property " + propName + " not set", actualValue);
//		assertEquals(message + "Property " + propName, expectedValue, actualValue);
//		
//		jmxc1.close();
//		
//		return actualValue;
//	}
    
    	public int getQueueDepth(Queue queue) throws JMSException {
		
            QueueConnection queueConnection = txTestQueueConnectionFactory.createQueueConnection();
            QueueSession queueSession = queueConnection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
            QueueBrowser queueBrowser = queueSession.createBrowser(queue);
            int numMessages = 0;
            for (Enumeration enumeration = queueBrowser.getEnumeration(); enumeration.hasMoreElements(); enumeration.nextElement()) {
                numMessages++;			
            }
            queueConnection.close();
            return numMessages;	
	}
    
        public void assertEquals(String message,int expected,int actual) throws Exception{
            if (expected!=actual){
                throw new Exception("Assertion failure: "+message+" expected="+expected+", actual="+actual);
            }
        }
}
